package com.muyu.pay.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.enums.SystemYesNo;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.nacos.service.NacosServerService;
import com.muyu.pay.domain.OrderPayCustomer;
import com.muyu.pay.domain.OrderPayInfo;
import com.muyu.pay.domain.req.CustomerListReq;
import com.muyu.pay.domain.resp.CustomerResp;
import com.muyu.pay.mapper.OrderPayCustomerMapper;
import com.muyu.pay.service.OrderPayCustomerService;
import com.muyu.pay.service.OrderPayService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.rmi.ServerException;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Classname OrderPayCustomerServiceImpl
 * @Description TODO
 * @Date 2024/8/9 上午9:09
 * @Created by 杨旭飞
 */
@Log4j2
@Service
public class OrderPayCustomerServiceImpl
        extends ServiceImpl<OrderPayCustomerMapper, OrderPayCustomer>
        implements OrderPayCustomerService {

    @Autowired
    private OrderPayService orderPayService;

    @Autowired
    private NacosServerService nacosServerService;

    /**
     * 查询客户集合
     * @param customerListReq 请求参数
     * @return 客户集合
     */
    @Override
    public List<CustomerResp> selectList(CustomerListReq customerListReq) {
        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(
                StringUtils.isNotEmpty(customerListReq.getAppName()),
                OrderPayCustomer::getAppName, customerListReq.getAppName()
        );
        queryWrapper.like(
                StringUtils.isNotEmpty(customerListReq.getAppCode()),
                OrderPayCustomer::getAppCode, customerListReq.getAppCode()
        );
        queryWrapper.eq(
                StringUtils.isNotEmpty(customerListReq.getStatus()),
                OrderPayCustomer::getStatus, customerListReq.getStatus()
        );
        List<OrderPayCustomer> orderPayCustomerList = this.list(queryWrapper);
        return orderPayCustomerList.stream()
                .map(orderPayCustomer -> CustomerResp.customerBuild(
                                orderPayCustomer,
                                () -> orderPayService.selectOrderPayByAppCodeAndLimit(orderPayCustomer.getAppCode(), 5)
                                        .stream()
                                        .map(OrderPayInfo::buildCustomerOrderPaySimpleResp)
                                        .toList()
                        )
                )
                .toList();
    }

    /**
     * 获取所有的客户
     * @return 客户集合
     */
    @Override
    public List<String> getCustomerAllList() {
        List<String> nacosServerAllList = nacosServerService.nacosServerAllList();
        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select(OrderPayCustomer::getAppCode);
        List<OrderPayCustomer> orderPayCustomerList = this.list(queryWrapper);
        Set<String> customerSet
                = orderPayCustomerList.stream().map(OrderPayCustomer::getAppCode).collect(Collectors.toSet());

        return nacosServerAllList.stream()
                .filter(nacosServer -> ! customerSet.contains(nacosServer))
                .toList();
    }

    @Override
    public boolean save(OrderPayCustomer orderPayCustomer) {
        String appCode = orderPayCustomer.getAppCode();

        List<String> nacosServerAllList = nacosServerService.nacosServerAllList();
        log.info("进行服务合法性判断：[{}——>{}]", appCode, nacosServerAllList);
        if(! nacosServerAllList.contains(appCode)) {
            throw new ServiceException("客户编码违法");
        }

        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderPayCustomer::getAppCode, appCode);
        long isAppCodeOnly = this.count(queryWrapper);
        log.info("进行服务code唯一性校验：[{}-{}]", appCode, isAppCodeOnly);
        if(isAppCodeOnly > 0) {
            throw new ServiceException("客户编码重复");
        }

        return super.save(orderPayCustomer);
    }

    /**
     * 禁用客户
     * @param orderCustomerId 支付客户ID
     */
    @Override
    public void disable(Long orderCustomerId) {
        this.settingStatus(orderCustomerId, SystemYesNo.NO.getCode());
    }

    /**
     * 启用客户
     * @param orderCustomerId 支付客户ID
     */
    @Override
    public void enable(Long orderCustomerId) {
        this.settingStatus(orderCustomerId, SystemYesNo.YES.getCode());
    }

    /**
     * 通过客户ID设置客户状态
     * @param orderCustomerId 支付客户ID
     * @param status 状态
     */
    @Override
    public void settingStatus(Long orderCustomerId, String status) {
        LambdaUpdateWrapper<OrderPayCustomer> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(OrderPayCustomer::getId, orderCustomerId);
        updateWrapper.set(OrderPayCustomer::getStatus, status);
        this.update(updateWrapper);
    }
}